/* Header file to declare function prototypes for cujmovie.c

   Please cite the C/C++ Users Journal article 
      A 3D Stereo Movie Viewer with GLUT
      Sept 2003
      Chris Moth
   in any derivatives of this work.

 *  Current contact information:
 *
 *    Chris Moth
 *    chris.moth@vanderbilt.edu
 *    Phone: 615-936-3569
 *    http://www.structbio.vanderbilt.edu/~cmoth
 */

// dispui.c call will call the functions below to 
// get the first and last frame numbers.
int displayMovieMinFrame(void);
int displayMovieMaxFrame(void);

// dispui.c will call this function when it needs to draw
// the scene
void displayRenderFrameInPerspectiveBox(int frame,int smearFlag);

// After you have rendered your scene in Perspective, dispui will call this
// function to render in an Ortho box.  This is particluarly useful if you
// want to show some text information in the movie window at fixed positions.

// For the CUJ demo, we do nothing.
void displayRenderFrameInOrthoBox(
   int frame,
   int movieWindowWidth,
   int movieWindowHeight);

// dispui.c will call this function when it gets a mouse click
// and needs a list of objects near the mouse click.
void displayRenderFrameInSelectMode(int frame);

// dispui.c will call this function when the clipping
// control is active.  In theory you could save
// your main window as an openGL displayList or use
// other tricks to speed up rendering of the clipping window.
// Here, I jsut draw the whole scene again.
void displayRenderClipToolFrame(int frame);

void displayProcessHits(
   GLint hits,             // The number of objects "near" our mouse click
   const GLuint* selectBuf // 4 GLuints for each hit
                        );

// Called by dispui when a new color input
void displayNewColorRequested(int _newColor);

void CenterOfRotationInputComplete(int lastKey,const char* inputString);

// dispui.c calls this function when the user presses <O>rigin
// or clicks on the origin box.
void displayCenterOfRotationRequested(void);

// You can use this function below to respond to clicks in the movie 
// Window without going into object selection mode.  This is especially useful
// if you have 2D text on the movie window, which might be removed or highlighted 
// when clicked.
int displayProcessRawMovieClick(int mouseX0,int mouseY0,int movieWindowWidth,int movieWindowHeight);

// When the user selects the staticMenu, dispui calls this function to let us
// know about it
void staticMenuCallback(int whichOption);

// Use this function to capture other keystrokes that
// are not part of the basic user interface.  This function is
// called by dispui when the keystrkoe command is not otherwise
// recognized
void displayKeyboardFunc(int key);

//  This function is called by dispui.c when the user 
// presses '?' or clicks on the help box.
void displayRenderHelpBox(void);
