package docview.views;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.internal.Workbench;
import org.eclipse.ui.part.*;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.graphics.Image;
import org.eclipse.jface.action.*;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.*;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.swt.SWT;

public class SampleView extends ViewPart {
	private TableViewer viewer;
	private Action action1;
	private Action action2;
	private Action doubleClickAction;

	private Map mapEditors = new HashMap();
	
	class windowListener implements IWindowListener {
		
		/**
		 * @see org.eclipse.ui.IWindowListener#windowActivated(IWorkbenchWindow)
		 */
		public void windowActivated(IWorkbenchWindow window) { 
		}

		/**
		 * @see org.eclipse.ui.IWindowListener#windowClosed(IWorkbenchWindow)
		 */
		public void windowClosed(IWorkbenchWindow window) {
			window.removePageListener(SampleView.this.pageEvents);
		}

		/**
		 * @see org.eclipse.ui.IWindowListener#windowDeactivated(IWorkbenchWindow)
		 */
		public void windowDeactivated(IWorkbenchWindow window) { }

		/**
		 * @see org.eclipse.ui.IWindowListener#windowOpened(IWorkbenchWindow)
		 */
		public void windowOpened(IWorkbenchWindow window) {
			window.addPageListener(SampleView.this.pageEvents);
		}

	}
	
	class pageListener implements IPageListener {
		
		/**
		 * @see org.eclipse.ui.IPageListener#pageActivated(IWorkbenchPage)
		 */
		public void pageActivated(IWorkbenchPage page) { }

		/**
		 * @see org.eclipse.ui.IPageListener#pageClosed(IWorkbenchPage)
		 */
		public void pageClosed(IWorkbenchPage page) {
			page.removePartListener(SampleView.this.partEvents);
		}

		/**
		 * @see org.eclipse.ui.IPageListener#pageOpened(IWorkbenchPage)
		 */
		public void pageOpened(IWorkbenchPage page) {
			page.addPartListener(SampleView.this.partEvents);
		}

	}
	
	class partListener implements IPartListener {
		
		/**
		 * @see org.eclipse.ui.IPartListener#partActivated(IWorkbenchPart)
		 */
		public void partActivated(IWorkbenchPart part) {
			if (SampleView.this.mapEditors.containsKey(part.toString())) {
				SampleView.this.viewer.setSelection(new StructuredSelection(SampleView.this.mapEditors.get(part.toString())), true);
			}
		}

		/**
		 * @see org.eclipse.ui.IPartListener#partBroughtToTop(IWorkbenchPart)
		 */
		public void partBroughtToTop(IWorkbenchPart part) {
			SampleView.this.viewer.refresh();
		}

		/**
		 * @see org.eclipse.ui.IPartListener#partClosed(IWorkbenchPart)
		 */
		public void partClosed(IWorkbenchPart part) {
			// the view itself is being released
			if (part == SampleView.this) {
				// so drop the listners
				IWorkbenchWindow[] windows = PlatformUI.getWorkbench().getWorkbenchWindows();
				for (int currWindow = 0; currWindow < windows.length; currWindow++) {
					windows[currWindow].removePageListener(SampleView.this.pageEvents);
					IWorkbenchPage pages[] = windows[currWindow].getPages();
					for (int currPage = 0; currPage < pages.length; currPage++) {
						pages[currPage].removePartListener(SampleView.this.partEvents);
					}
				}
			} else if (SampleView.this.mapEditors.containsKey(part.toString())) {
				SampleView.this.mapEditors.remove(part.toString());
				SampleView.this.viewer.refresh();
			}
		}

		/**
		 * @see org.eclipse.ui.IPartListener#partDeactivated(IWorkbenchPart)
		 */
		public void partDeactivated(IWorkbenchPart part) { }

		/**
		 * @see org.eclipse.ui.IPartListener#partOpened(IWorkbenchPart)
		 */
		public void partOpened(IWorkbenchPart part) {
			if (part instanceof IEditorPart) {
				String key = part.toString();
				if (! SampleView.this.mapEditors.containsKey(part.toString())) {
					SampleView.this.mapEditors.put(part.toString(), part);
					SampleView.this.viewer.refresh();
				}
			}
		}
	}
	
	private windowListener windowEvents;
	private pageListener pageEvents;
	private partListener partEvents;
	  
	class ViewContentProvider implements IStructuredContentProvider {
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}
		public void dispose() {
		}
		public Object[] getElements(Object parent) {
			return mapEditors.values().toArray();
		}
	}
	class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {
		public String getColumnText(Object obj, int index) {
			if (obj instanceof IEditorPart) {
				return ((IEditorPart) obj).getTitle(); 
			}
			else {
				return "Object not a reference to an editor: " + obj.toString();
			}
		}

		public Image getColumnImage(Object obj, int index) {
			return getImage(obj);
		}
		
		public Image getImage(Object obj) {
			if (obj instanceof IEditorPart) {
				return ((IEditorPart) obj).getTitleImage();
			}
			else {
				return super.getImage(obj);
			}
		}
	}

	class NameSorter extends ViewerSorter {
	}

	/**
	 * The constructor.
	 */
	public SampleView() {
		windowEvents = new windowListener();
		partEvents = new partListener();
		pageEvents = new pageListener();
		
		PlatformUI.getWorkbench().addWindowListener(windowEvents);

	}


	private void xinitMap() {
		PlatformUI.getWorkbench().addWindowListener(windowEvents);
		IWorkbenchWindow[] windows = PlatformUI.getWorkbench().getWorkbenchWindows();
		for (int currWindow = 0; currWindow < windows.length; currWindow++) {
			windows[currWindow].addPageListener(pageEvents);
			IWorkbenchPage pages[] = windows[currWindow].getPages();
			for (int currPage = 0; currPage < pages.length; currPage++) {
				pages[currPage].addPartListener(partEvents);
				IEditorReference editors[] = pages[currPage].getEditorReferences();
				for (int currEditor = 0; currEditor < editors.length; currEditor++) {
					IEditorPart editor = editors[currEditor].getEditor(true);
					mapEditors.put(editor.toString(), editor);
				}
			}
		}
	}

	/**
	 * This is a callback that will allow us
	 * to create the viewer and initialize it.
	 */
	public void createPartControl(Composite parent) {


		viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		viewer.setContentProvider(new ViewContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider());
		viewer.setSorter(new NameSorter());
		viewer.setInput(ResourcesPlugin.getWorkspace());
		
		makeActions();
		hookContextMenu();
		hookDoubleClickAction();
		contributeToActionBars();


		// when the system is starting, the object model won't be ready for us to 
		// traverse, so create a thread the syncs on the isStarting flag.  
		// there isn't a callback at the end of start-up, so this will have to do. 
		Runnable operation = new Runnable() {
			public void run() {
				Workbench wb = (Workbench) PlatformUI.getWorkbench();
				while (wb.isStarting()) 
					try {
						Thread.sleep(500);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					PlatformUI.getWorkbench().addWindowListener(windowEvents);
					IWorkbenchWindow[] windows = PlatformUI.getWorkbench().getWorkbenchWindows();
					for (int currWindow = 0; currWindow < windows.length; currWindow++) {
						windows[currWindow].addPageListener(pageEvents);
						IWorkbenchPage pages[] = windows[currWindow].getPages();
						for (int currPage = 0; currPage < pages.length; currPage++) {
							pages[currPage].addPartListener(partEvents);
							IEditorReference editors[] = pages[currPage].getEditorReferences();
							for (int currEditor = 0; currEditor < editors.length; currEditor++) {
								IEditorPart editor = editors[currEditor].getEditor(true);
								mapEditors.put(editor.toString(), editor);
							}
						}
					}
				SampleView.this.viewer.refresh();
			}
		};
		Display.getCurrent().asyncExec(operation);

		PlatformUI.getWorkbench().addWindowListener(windowEvents);
	}

	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				SampleView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalPullDown(IMenuManager manager) {
		manager.add(action1);
		manager.add(new Separator());
		manager.add(action2);
	}

	private void fillContextMenu(IMenuManager manager) {
		manager.add(action1);
		manager.add(action2);
		// Other plug-ins can contribute there actions here
		manager.add(new Separator("Additions"));
	}
	
	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(action1);
		manager.add(action2);
	}

	private void makeActions() {
		action1 = new Action() {
			public void run() {
				ISelection selection = viewer.getSelection();
				for (Iterator iter = ((IStructuredSelection) selection).iterator(); iter.hasNext();) {
					IEditorPart currEditor = (IEditorPart) iter.next();
					if (currEditor.isDirty()) {
						currEditor.doSave(null);
					}
				}
			}
		};

		action1.setText("Save");
		action1.setToolTipText("Write the contents of this editor to disk");
		action1.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().
			getImageDescriptor(ISharedImages.IMG_OBJ_FILE));
		
		action2 = new Action() {
			public void run() {
				ISelection selection = viewer.getSelection();
				IEditorPart currEditor = (IEditorPart) ((IStructuredSelection)selection).getFirstElement();
				currEditor.getEditorSite().getPage().activate(currEditor);
			}
		};
		action2.setText("Switch to");
		action2.setToolTipText("Switch to this editor");
		action2.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().
			getImageDescriptor(ISharedImages.IMG_OBJS_TASK_TSK));
		doubleClickAction = action2;
	}

	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				doubleClickAction.run();
			}
		});
	}
	private void showMessage(String message) {
		MessageDialog.openInformation(
			viewer.getControl().getShell(),
			"Sample View",
			message);
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		viewer.getControl().setFocus();
		System.out.println("setFocus");
	}

}