/* /////////////////////////////////////////////////////////////////////////////
 * File:        int_to_string_test.cpp
 *
 * Purpose:     Implementation file for the int_to_string_test project.
 *
 * Created:     15th July 2003
 * Updated:     16th July 2003
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2003, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2003
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

//#define	_WINSTL_INT_TO_STRING_USE_DECLSPECTHREAD_FOR_EXES
//#define	__STLSOFT_INTEGER_TO_STRING_OPTIMISE_64BIT
//#define	USE_UINT64

#include <windows.h>
#include <stdio.h>

#include <stlsoft.h>

#include <winstl.h>

#if defined(__STLSOFT_COMPILER_IS_BORLAND)
 #include <winstl_tick_counter.h>

 typedef winstl_ns_qual(tick_counter)			performance_counter;
#else
 #include <winstl_performance_counter.h>

 typedef winstl_ns_qual(performance_counter)	performance_counter;
#endif /* compiler */

#include <winstl_int_to_string.h>

#ifdef TEST_SYNESISSTD_I2S

#include <MLNumStr.h>

using SynesisStd::LongToStringA;
using SynesisStd::ULLongToStringA;

#endif /* TEST_SYNESISSTD_I2S */

////////////////////////////////////////////////////////////////////////////////
// Constants and definitions

#ifdef USE_UINT64
 typedef	stlsoft::ss_uint64_t	int_type;
#else
 typedef	stlsoft::ss_sint32_t	int_type;
#endif /* USE_UINT64 */

/* long long printf format sequence */

#if defined(__STLSOFT_COMPILER_IS_BORLAND)
 #define _res_fmt		"%I64d"
 #define _uint64_fmt    "%I64u"
#elif defined(__STLSOFT_COMPILER_IS_INTEL) || \
      defined(__STLSOFT_COMPILER_IS_MSVC)
 #define _res_fmt		"%I64d"
 #define _uint64_fmt    "%I64u"
#elif defined(__STLSOFT_COMPILER_IS_DMC) || \
      defined(__STLSOFT_COMPILER_IS_GCC) || \
      defined(__STLSOFT_COMPILER_IS_MWERKS)
 #define _res_fmt		"%lld"
 #define _uint64_fmt    "%llu"
#else
 #error Further compiler discrimination is required
#endif /* compiler */


#ifdef _DEBUG
const int DEF_ITERATIONS	=   1000;
#else
const int DEF_ITERATIONS    =   10000000;
#endif /* _DEBUG */

/* ////////////////////////////////////////////////////////////////////////// */

int test_sprintf(int iterations);
int test_integer_to_string(int iterations);
int test_int_to_string(int iterations);
#ifdef TEST_SYNESISSTD_I2S
int test_IntToString(int iterations);
#endif /* TEST_SYNESISSTD_I2S */

/* ////////////////////////////////////////////////////////////////////////// */

DWORD WINAPI ThreadProc(LPVOID /* pv */)
{
	OutputDebugStringA("Thread entered\n");

	::Sleep(30000);

	OutputDebugStringA("Thread exited\n");

	return 0;
}

int main(int argc, char *argv[])
{
    int                 iRet    =   0;
    int                 i;
	char const			*pcszIterations	=	NULL;

	winstl::int_to_string<char>(stlsoft::ss_sint16_t(0));
	winstl::int_to_string<char>(stlsoft::ss_uint32_t(0));

	winstl::int_to_string<wchar_t>(stlsoft::ss_uint16_t(0));
	winstl::int_to_string<wchar_t>(stlsoft::ss_sint32_t(0));

#ifdef _DEBUG
	for(i = 0; i < 4; ++i)
	{
		DWORD	tid;

		::CreateThread(NULL, 0, ThreadProc, NULL, 0, &tid);
	}
#endif /* _DEBUG */

	::Sleep(10);

	for(i = 1; i < argc; ++i)
	{
		const char	*arg	=	argv[i];

		if(arg[0] == '-')
		{
			if(arg[1] == '-')
			{
				/* -- arguments */
			}
			else
			{
				/* - arguments */
			}
		}
		else
		{
			/* other arguments */
			pcszIterations = arg;
		}
	}

	int	iterations	=	(NULL != pcszIterations) ? atoi(pcszIterations) : DEF_ITERATIONS;

	printf("int_to_string_test (%s):  %d iterations\n", __STLSOFT_COMPILER_LABEL_STRING, iterations);

	::SetThreadPriority(::GetCurrentThread(), THREAD_PRIORITY_HIGHEST);

	iRet += test_sprintf(iterations);
	iRet += test_integer_to_string(iterations);
	iRet += test_int_to_string(iterations);
#ifdef TEST_SYNESISSTD_I2S
	iRet += test_IntToString(iterations);
#endif /* TEST_SYNESISSTD_I2S */

    return iRet;
}

/* ////////////////////////////////////////////////////////////////////////// */

int test_integer_to_string(int iterations)
{
    performance_counter counter;
	int					iRet	=	0;

	for(int WARMUPS = 2; WARMUPS; --WARMUPS)
	{
		iRet = 0;
		counter.start();
		for(int_type i = 0; i < (int_type)iterations; ++i)
		{
			char		buffer[21];
			char const	*s	=	stlsoft::integer_to_string(buffer, stlsoft_num_elements(buffer), i);

			iRet += (int)*s;
		}
		counter.stop();
	}

	printf("integer_to_string: " _res_fmt "ms\n", counter.get_milliseconds());

	return iRet;
}

int test_int_to_string(int iterations)
{
    performance_counter counter;
	int					iRet	=	0;

	for(int WARMUPS = 2; WARMUPS; --WARMUPS)
	{
		iRet = 0;
		counter.start();
		for(int_type i = 0; i < (int_type)iterations; ++i)
		{
			char const	*s	=	winstl::int_to_string<char>(i);

			iRet += (int)*s;
		}
		counter.stop();
	}

	printf("int_to_string:     " _res_fmt "ms\n", counter.get_milliseconds());

	return iRet;
}

#ifdef TEST_SYNESISSTD_I2S
int test_IntToString(int iterations)
{
    performance_counter counter;
	int					iRet	=	0;

	for(int WARMUPS = 2; WARMUPS; --WARMUPS)
	{
		iRet = 0;
		counter.start();
		for(int_type i = 0; i < (int_type)iterations; ++i)
		{
#ifdef USE_UINT64
			char const	*s	=	ULLongToStringA(i);
#else
			char const	*s	=	LongToStringA(i);
#endif /* USE_UINT64 */

			iRet += (int)*s;
		}
		counter.stop();
	}

	printf("IntToString:       " _res_fmt "ms\n", counter.get_milliseconds());

	return iRet;
}
#endif /* TEST_SYNESISSTD_I2S */

int test_sprintf(int iterations)
{
    performance_counter counter;
	int					iRet	=	0;

	for(int WARMUPS = 2; WARMUPS; --WARMUPS)
	{
		iRet = 0;
		counter.start();
		for(int_type i = 0; i < (int_type)iterations; ++i)
		{
			char		buffer[21];

#ifdef USE_UINT64
			sprintf(buffer, _uint64_fmt, i);
#else
			sprintf(buffer, "%ld", i);
#endif /* USE_UINT64 */

			iRet += (int)*buffer;
		}
		counter.stop();
	}

//	printf(

	printf("sprintf:           " _res_fmt "ms\n", counter.get_milliseconds());

	return iRet;
}

/* ////////////////////////////////////////////////////////////////////////// */

