/* /////////////////////////////////////////////////////////////////////////////
 * File:        tlsmax.c
 *
 * Purpose:     TLS capacity test program.
 *
 * Created:     27th August 2001
 * Updated:     17th July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Private Supply License)
 *
 *              Copyright (C) 2002, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *                          software@synesis-group.com
 *
 *              Use in source and binary forms, and redistribution in binary
 *              forms, without modification, are permitted provided that the
 *              following conditions are met:
 *
 *              (i) Redistributions of source code is prohibited without
 *              specific written consent from the original supplier.
 *              Redistributions of source code must retain the above copyright
 *              notice and contact information, this list of conditions and the
 *              following disclaimer.
 *
 *              (ii) Redistribution in binary form is subject to any
 *              non-redistribution stipulations provided by Synesis Software, or
 *              passed on by an intermediate supplier, at the time of
 *              distribution. Specifically, if a document stipulating one or
 *              more binary components are NOT to be redistributed is obtained
 *              from the supplier at the time of distribution then the user must
 *              (a) not redistribute any material named in that document, and
 *              (b) must redistribute that document with any other materials
 *              redistributed.
 *
 *              Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              (vi) Prominent citations stating the name of the Synesis
 *              Software are placed into the source-code, executable programs
 *              and any associated literature of any work based on or using the
 *              software. Citations in the executables programs must be in the
 *              About Box (or similar concept) of a program interacting with the
 *              user via a Graphical User Interface (GUI), or in the form of an
 *              information string to be printed to the console along with other
 *              help and version information of a non-GUI or command-line
 *              program.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *               warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* The absence of any include files is not just caprice, but a paranoid way
 * of ensuring that absolutely nothing gets done (e.g. within a static function
 * or a static class instance) that might involve TLS allocation.
 *
 * The same reasoning is the justification for why USER32.lib is not linked, so
 * as to avoid any TLS allocation it may do in its initialisation.
 *
 * The figures derived can therefore be seen to reflect the maximum conceivable
 * capacity for any process of the given system.
 */

#define	SLOT_MAX_TRY	(1000000)

#define	NUM_ELEMENTS(x)	(sizeof(x) / sizeof(x[0]))

unsigned long	__stdcall	TlsAlloc(void);
void			__stdcall	TlsFree(unsigned long);
unsigned long	__stdcall	GetLastError(void);
void			__stdcall	SetLastError(unsigned long);
void			*__stdcall	LoadLibraryA(char const *);
void			*__stdcall	GetProcAddress(void *, char const *);

int __stdcall WinMain(	void	*hinst,
						void	*hinstPrev,
						char	*lpszCmdLine,
						int		nCmdShow)
{
	int				i;
	int				j;
	unsigned long	slot;
	unsigned long	previous_slots[10];	/* We'll give back 10 slots to USER32.dll */

	for(j = 0; j < NUM_ELEMENTS(previous_slots); ++j)
	{
		previous_slots[j] = 0xFFFFFFFF;
	}

	SetLastError(0);

	for(i = 0; i < SLOT_MAX_TRY; ++i)
	{
		slot = TlsAlloc();

		if(	0xFFFFFFFF == slot/*  ||
			0 != GetLastError() */)
		{
			break;
		}

		for(j = 0; j < NUM_ELEMENTS(previous_slots) - 1; ++j)
		{
			previous_slots[j + 1] = previous_slots[j];
		}
		previous_slots[0] = slot;
	}

	for(j = 0; j < NUM_ELEMENTS(previous_slots) - 1; ++j)
	{
		if(0xFFFFFFFF != previous_slots[j])
		{
			TlsFree(previous_slots[j]);
		}
	}

	// Now load MessageBox(), & wsprintf() and display
	{
		typedef int (__stdcall *PFnMessageBoxA)(void *, char const *, char const *, int);
		typedef int (__cdecl *PFnwsprintfA)(char *, char const *, ...);

		char			message[101];
		void			*hUser32	=	LoadLibraryA("USER32");
		PFnMessageBoxA	MessageBox	=	(PFnMessageBoxA)GetProcAddress(hUser32, "MessageBoxA");
		PFnwsprintfA	wsprintfA	=	(PFnwsprintfA)GetProcAddress(hUser32, "wsprintfA");

		if(i < SLOT_MAX_TRY)
		{
			wsprintfA(message, "%d slots", i);
		}
		else
		{
			wsprintfA(message, "%d or more slots were acquired", SLOT_MAX_TRY);
		}

		MessageBox(0, message, "TLS Slot Capacity", 0);
	}

	return 0;
}
